@@ -20,7 +20,7 @@ import logging
 import io
 import os
 
-from os.path import dirname, lexists, realpath
+from os.path import dirname, lexists, realpath, sep as pathsep
 from os.path import exists
 from os.path import isabs
 from os.path import join as opj
@@ -30,6 +30,10 @@ from .utils import getpwd
 from .dochelpers import exc_str
 from .support.annexrepo import AnnexRepo
 from .cmdline.helpers import get_repo_instance
+from .consts import HANDLE_META_DIR
+
+# To be used for a quick detection of path being under .git/
+_DOT_GIT_DIR = pathsep + '.git' + pathsep
 
 lgr = logging.getLogger("datalad.auto")
 
@@ -50,6 +54,9 @@ except Exception as exc:
 
 lzma = None
 try:
+    # Our mocking would not work with backports.lzma ATM, so only lzma
+    # would be supported
+    # from datalad.support.lzma import lzma
     import lzma
 except ImportError:
     pass
@@ -62,8 +69,10 @@ except Exception as exc:
 # TODO: RF to reduce code duplication among cases, also RF tests for the same reason
 
 class _EarlyExit(Exception):
-    """Helper to early escape try/except logic in wrappde open"""
-    pass
+    """Helper to early escape try/except logic in wrapped open"""
+    def __init__(self, msg, *args):
+        self.msg = msg
+        self.args = args
 
 
 class AutomagicIO(object):
@@ -72,10 +81,25 @@ class AutomagicIO(object):
     Currently supports builtin open() and h5py.File when those are read
     """
 
-    def __init__(self, autoget=True, activate=False):
+    def __init__(self, autoget=True, activate=False, check_once=False):
+        """
+        
+        Parameters
+        ----------
+        autoget
+        activate
+        check_once: bool, optional
+          To speed things up and avoid unnecessary repeated checks, 
+          if True, paths considered for proxying and corresponding repositories
+          are remembered, and are not subject to datalad checks on subsequent calls.
+          This option is to be used if you do not expect new git repositories to not
+          be created and files not to get dropped while operating under 
+          AutomagicIO supervision.
+        """
         self._active = False
         self._builtin_open = __builtin__.open
         self._io_open = io.open
+        self._os_stat = os.stat
         self._builtin_exists = os.path.exists
         self._builtin_isfile = os.path.isfile
         if h5py:
@@ -91,6 +115,8 @@ class AutomagicIO(object):
         self._log_online = True
         from mock import patch
         self._patch = patch
+        self._paths_cache = set() if check_once else None
+        self._repos_cache = {} if check_once else None
         if activate:
             self.activate()
 
@@ -116,12 +142,12 @@ class AutomagicIO(object):
         # wrap it all for resilience to errors -- proxying must do no harm!
         try:
             if self._in_open:
-                raise _EarlyExit
+                raise _EarlyExit("within open already")
             self._in_open = True  # just in case someone kept alias/assignment
             # return stock open for the duration of handling so that
             # logging etc could workout correctly
             with self._patch(origname, origfunc):
-                lgr.log(2, "Proxying open with %r %r", args, kwargs)
+                lgr.log(3, "Proxying open with %r %r", args, kwargs)
 
                 # had to go with *args since in PY2 it is name, in PY3 file
                 # deduce arguments
@@ -132,13 +158,21 @@ class AutomagicIO(object):
                     filearg = "name" if PY2 else "file"
                     if filearg not in kwargs:
                         # so the name was missing etc, just proxy into original open call and let it puke
-                        lgr.debug("No name/file was given, avoiding proxying")
-                        raise _EarlyExit
+                        raise _EarlyExit("no name/file was given")
                     file = kwargs.get(filearg)
+
                 if isinstance(file, int):
-                    lgr.debug(
-                        "Skipping operation on %i, already a file descriptor", file)
-                    raise _EarlyExit
+                    raise _EarlyExit("already a file descriptor")
+
+                if self._paths_cache is not None:
+                    filefull = file if isabs(file) else os.path.abspath(file)
+                    if filefull in self._paths_cache:
+                        raise _EarlyExit("considered before")
+                    else:
+                        self._paths_cache.add(filefull)
+
+                if _DOT_GIT_DIR in file:
+                    raise _EarlyExit("we ignore paths under .git/")
                 mode = 'r'
                 if len(args) > 1:
                     mode = args[1]
@@ -148,9 +182,10 @@ class AutomagicIO(object):
                 if 'r' in mode:
                     self._dataset_auto_get(file)
                 else:
-                    lgr.debug("Skipping operation on %s since mode=%r", file, mode)
-        except _EarlyExit:
-            pass
+                    raise _EarlyExit("mode=%r", mode)
+        except _EarlyExit as e:
+            lgr.log(2, " skipping since " + e.msg, *e.args,
+                    extra={'notraceback': True})
         except Exception as e:
             # If anything goes wrong -- we should complain and proceed
             with self._patch(origname, origfunc):
@@ -168,6 +203,10 @@ class AutomagicIO(object):
         return self._proxy_open_name_mode('io.open', self._io_open,
                                           *args, **kwargs)
 
+    def _proxy_os_stat(self, *args, **kwargs):
+        return self._proxy_open_name_mode('os.stat', self._os_stat,
+                                          *args, **kwargs)
+
     def _proxy_h5py_File(self, *args, **kwargs):
         return self._proxy_open_name_mode('h5py.File', self._h5py_File,
                                           *args, **kwargs)
@@ -195,21 +234,42 @@ class AutomagicIO(object):
             return
         # if filepath is not there at all (program just "checked" if it could access it
         if not lexists(filepath):
-            lgr.log(2, "Not testing/getting file %s since it is not there", filepath)
+            lgr.log(2, " skipping %s since it is not there", filepath)
             return
         # deduce directory for filepath
         filedir = dirname(filepath)
-        try:
-            # TODO: verify logic for create -- we shouldn't 'annexify' non-annexified
-            # see https://github.com/datalad/datalad/issues/204
-            annex = get_repo_instance(filedir)
-        except (RuntimeError, InvalidGitRepositoryError) as e:
-            # must be not under annex etc
-            return
+        annex = None
+        if self._repos_cache is not None:
+            filedir_parts = filedir.split(pathsep)
+            # ATM we do not expect subdatasets under .datalad, so we could take the top
+            # level dataset for that
+            try:
+                filedir = pathsep.join(
+                    filedir_parts[:filedir_parts.index(HANDLE_META_DIR)]
+                )
+            except ValueError:
+                # would happen if no .datalad
+                pass
+            try:
+                annex = self._repos_cache[filedir]
+            except KeyError:
+                pass
+
+        if annex is None:
+            try:
+                # TODO: verify logic for create -- we shouldn't 'annexify' non-annexified
+                # see https://github.com/datalad/datalad/issues/204
+                annex = get_repo_instance(filedir)
+                lgr.log(2, "Got the repository %s id:%s containing %s", annex, id(annex), filedir)
+            except (RuntimeError, InvalidGitRepositoryError) as e:
+                # must be not under annex etc
+                return
+            if self._repos_cache is not None:
+                self._repos_cache[filedir] = annex
         if not isinstance(annex, AnnexRepo):
             # not an annex -- can do nothing
+            lgr.log(2, " skipping %s since the repo is not annex", filepath)
             return
-
         # since Git/AnnexRepo functionality treats relative paths relative to the
         # top of the repository and might be outside, get a full path
         if not isabs(filepath):
@@ -225,7 +285,11 @@ class AutomagicIO(object):
         # either it has content
         if (under_annex or under_annex is None) and not annex.file_has_content(filepath):
             lgr.info("AutomagicIO: retrieving file content of %s", filepath)
-            annex.get(filepath)
+            out = annex.get(filepath)
+            if not out.get('success', False):
+                # to assure that it is present and without trailing/leading new lines
+                out['note'] = out.get('note', '').strip()
+                lgr.error("Failed to retrieve %(file)s: %(note)s", out)
 
     def activate(self):
         # we should stay below info for this message. With PR #1630 we
@@ -251,6 +315,7 @@ class AutomagicIO(object):
         # overloads
         __builtin__.open = self._proxy_open
         io.open = self._proxy_io_open
+        os.stat = self._proxy_os_stat
         os.path.exists = self._proxy_exists
         os.path.isfile = self._proxy_isfile
         if h5py:
@@ -267,6 +332,7 @@ class AutomagicIO(object):
             return
         __builtin__.open = self._builtin_open
         io.open = self._io_open
+        os.stat = self._os_stat
         if h5py:
             h5py.File = self._h5py_File
         if lzma:
