@@ -93,7 +93,7 @@ def _parse_env(store):
     for k in os.environ:
         if not k.startswith('DATALAD_'):
             continue
-        dct[k.replace('_', '.').lower()] = os.environ[k]
+        dct[k.replace('__', '-').replace('_', '.').lower()] = os.environ[k]
     store.update(dct)
     return store
 
@@ -421,6 +421,7 @@ class ConfigManager(object):
         """Returns list of configuration item names"""
         return self._store.keys()
 
+    # XXX should this be *args?
     def get(self, key, default=None):
         """D.get(k[,d]) -> D[k] if k in D, else d.  d defaults to None."""
         return self._store.get(key, default)
