@@ -34,6 +34,7 @@ from datalad.support.gitrepo import InvalidGitRepositoryError
 from datalad.support.gitrepo import NoSuchPathError
 from datalad.support.repo import Flyweight
 from datalad.support.network import RI
+from datalad.support.exceptions import InvalidAnnexRepositoryError
 
 from datalad.utils import getpwd
 from datalad.utils import optional_args, expandpath, is_explicit_path
@@ -161,6 +162,7 @@ class Dataset(object):
         self._repo = None
         self._id = None
         self._cfg = None
+        self._cfg_bound = None
 
     def __repr__(self):
         return "<Dataset path=%s>" % self.path
@@ -187,22 +189,61 @@ class Dataset(object):
     @property
     def repo(self):
         """Get an instance of the version control system/repo for this dataset,
-        or None if there is none yet.
+        or None if there is none yet (or none anymore).
 
-        If creating an instance of GitRepo is guaranteed to be really cheap
-        this could also serve as a test whether a repo is present.
+        If testing the validity of an instance of GitRepo is guaranteed to be
+        really cheap this could also serve as a test whether a repo is present.
+
+        Note, that this property is evaluated every time it is used. If used
+        multiple times within a function it's probably a good idea to store its
+        value in a local variable and use this variable instead.
 
         Returns
         -------
-        GitRepo
+        GitRepo or AnnexRepo
         """
 
-        # Note: lazy loading was disabled, since this is provided by the
-        # flyweight pattern already and a possible invalidation of an existing
-        # instance has to be done therein.
-        # TODO: Still this is somewhat problematic. We can't invalidate strong
-        # references
+        # If we already got a *Repo instance, check whether it's still valid;
+        # Note, that this basically does part of the testing that would
+        # (implicitly) be done in the loop below again. So, there's still
+        # potential to speed up when we actually need to get a new instance
+        # (or none). But it's still faster for the vast majority of cases.
+        #
+        # TODO: Dig deeper into it and melt with new instance guessing. This
+        # should also involve to reduce redundancy of testing such things from
+        # within Flyweight.__call__, AnnexRepo.__init__ and GitRepo.__init__!
+        #
+        # Also note, that this could be forged into a single big condition, but
+        # that is hard to read and we should be well aware of the actual
+        # criteria here:
+        if self._repo is not None and realpath(self.path) == self._repo.path:
+            # we got a repo and path references still match
+            if isinstance(self._repo, AnnexRepo):
+                # it's supposed to be an annex
+                if self._repo is AnnexRepo._unique_instances.get(
+                        self._repo.path, None) and \
+                        AnnexRepo.is_valid_repo(self._repo.path,
+                                                allow_noninitialized=True):
+                    # it's still the object registered as flyweight and it's a
+                    # valid annex repo
+                    return self._repo
+            elif isinstance(self._repo, GitRepo):
+                # it's supposed to be a plain git
+                if self._repo is GitRepo._unique_instances.get(
+                        self._repo.path, None) and \
+                        GitRepo.is_valid_repo(self._repo.path) and not \
+                        self._repo.is_with_annex():
+                    # it's still the object registered as flyweight, it's a
+                    # valid git repo and it hasn't turned into an annex
+                    return self._repo
 
+        # Note: Although it looks like the "self._repo = None" assignments
+        # could be used instead of variable "valid", that's a big difference!
+        # The *Repo instances are flyweights, not singletons. self._repo might
+        # be the last reference, which would lead to those objects being
+        # destroyed and therefore the constructor call would result in an
+        # actually new instance. This is unnecessarily costly.
+        valid = False
         for cls, ckw, kw in (
                 # TODO: Do we really want to allow_noninitialized=True here?
                 # And if so, leave a proper comment!
@@ -213,13 +254,17 @@ class Dataset(object):
                 try:
                     lgr.log(5, "Detected %s at %s", cls, self._path)
                     self._repo = cls(self._path, create=False, **kw)
+                    valid = True
                     break
-                except (InvalidGitRepositoryError, NoSuchPathError) as exc:
+                except (InvalidGitRepositoryError, NoSuchPathError,
+                        InvalidAnnexRepositoryError) as exc:
                     lgr.log(5,
                             "Oops -- guess on repo type was wrong?: %s",
                             exc_str(exc))
-                    pass
-                # version problems come as RuntimeError: DO NOT CATCH!
+
+        if not valid:
+            self._repo = None
+
         if self._repo is None:
             # Often .repo is requested to 'sense' if anything is installed
             # under, and if so -- to proceed forward. Thus log here only
@@ -236,31 +281,50 @@ class Dataset(object):
         for different versions of the same dataset (that have all been derived
         from the same original dataset repository).
 
+        Note, that a plain git/git-annex repository doesn't necessarily have
+        a dataset id yet. It is created by `Dataset.create()` and stored in
+        .datalad/config. If None is returned while there is a valid repository,
+        there may have never been a call to `create` in this branch before
+        current commit.
+
+        Note, that this property is evaluated every time it is used. If used
+        multiple times within a function it's probably a good idea to store its
+        value in a local variable and use this variable instead.
+
         Returns
         -------
         str
           This is either a stored UUID, or `None`.
         """
-        if self._id is None:
-            # if we have one on record, stick to it!
-            self.config.reload()
-            self._id = self.config.get('datalad.dataset.id', None)
-        return self._id
+
+        return self.config.get('datalad.dataset.id', None)
 
     @property
     def config(self):
         """Get an instance of the parser for the persistent dataset configuration.
 
+        Note, that this property is evaluated every time it is used. If used
+        multiple times within a function it's probably a good idea to store its
+        value in a local variable and use this variable instead.
+
         Returns
         -------
         ConfigManager
         """
-        if self._cfg is None:
-            if self.repo is None:
-                # associate with this dataset and read the entire config hierarchy
-                self._cfg = ConfigManager(dataset=self, dataset_only=False)
-            else:
-                self._cfg = self.repo.config
+
+        if self.repo is None:
+            # if there's no repo (yet or anymore), we can't read/write config at
+            # dataset level, but only at user/system level
+            # However, if this was the case before as well, we don't want a new
+            # instance of ConfigManager
+            if self._cfg_bound in (True, None):
+                self._cfg = ConfigManager(dataset=None, dataset_only=False)
+                self._cfg_bound = False
+
+        else:
+            self._cfg = self.repo.config
+            self._cfg_bound = True
+
         return self._cfg
 
     def get_subdatasets(self, pattern=None, fulfilled=None, absolute=False,
@@ -312,21 +376,9 @@ class Dataset(object):
         -------
         bool
         """
-        # do early check manually if path exists to not even ask git at all
-        exists_now = exists(self.path)
 
-        was_once_installed = None
-        if exists_now:
-            was_once_installed = self.path is not None and \
-                                 self.repo is not None
-
-        if not exists_now or \
-                (was_once_installed and not GitRepo.is_valid_repo(self.path)):
-            # repo gone now, reset
-            self._repo = None
-            return False
-        else:
-            return was_once_installed
+        return self.path is not None and exists(self.path) and \
+            self.repo is not None
 
     def get_superdataset(self, datalad_only=False, topmost=False,
                          registered_only=True):
@@ -335,7 +387,7 @@ class Dataset(object):
         Parameters
         ----------
         datalad_only : bool, optional
-          Either to consider only "datalad datasets" (with non-None
+          Whether to consider only "datalad datasets" (with non-None
           id), or (if False, which is default) - any git repository
         topmost : bool, optional
           Return the topmost super-dataset. Might then be the current one.
