@@ -25,12 +25,16 @@ from datalad.utils import on_windows
 from datalad.support.gitrepo import GitRepo
 from datalad.support.annexrepo import AnnexRepo
 
-from nose.tools import ok_, eq_, assert_false, assert_equal, assert_true, assert_is_instance
+from nose.tools import ok_, eq_, assert_false, assert_equal, assert_true, \
+    assert_is_instance, assert_is_none, assert_is_not, assert_is_not_none
 from datalad.tests.utils import SkipTest
 from datalad.tests.utils import with_tempfile, assert_in, with_tree, with_testrepos
 from datalad.tests.utils import assert_cwd_unchanged
 from datalad.tests.utils import assert_raises
 from datalad.tests.utils import known_failure_windows
+from datalad.tests.utils import assert_is
+from datalad.tests.utils import assert_not_equal
+
 from datalad.support.exceptions import InsufficientArgumentsError
 from datalad.support.exceptions import PathOutsideRepositoryError
 
@@ -323,3 +327,112 @@ def test_Dataset_flyweight(path1, path2):
             ds3 = Dataset('linked')
             ok_(ds3 == ds1)
             ok_(ds3 is not ds1)
+
+
+@with_tempfile
+def test_property_reevaluation(repo1):
+
+    from os.path import lexists
+    from datalad.tests.utils import ok_clean_git
+
+    ds = Dataset(repo1)
+    assert_is_none(ds.repo)
+    assert_is_not_none(ds.config)
+    first_config = ds.config
+    assert_false(ds._cfg_bound)
+    assert_is_none(ds.id)
+
+    ds.create()
+    ok_clean_git(repo1)
+    # after creation, we have `repo`, and `config` was reevaluated to point
+    # to the repo's config:
+    assert_is_not_none(ds.repo)
+    assert_is_not_none(ds.config)
+    second_config = ds.config
+    assert_true(ds._cfg_bound)
+    assert_is(ds.config, ds.repo.config)
+    assert_is_not(first_config, second_config)
+    assert_is_not_none(ds.id)
+    first_id = ds.id
+
+    ds.remove()
+    # repo is gone, and config is again reevaluated to only provide user/system
+    # level config:
+    assert_false(lexists(ds.path))
+    assert_is_none(ds.repo)
+    assert_is_not_none(ds.config)
+    third_config = ds.config
+    assert_false(ds._cfg_bound)
+    assert_is_not(second_config, third_config)
+    assert_is_none(ds.id)
+
+    ds.create()
+    ok_clean_git(repo1)
+    # after recreation everything is sane again:
+    assert_is_not_none(ds.repo)
+    assert_is_not_none(ds.config)
+    assert_is(ds.config, ds.repo.config)
+    forth_config = ds.config
+    assert_true(ds._cfg_bound)
+    assert_is_not(third_config, forth_config)
+    assert_is_not_none(ds.id)
+    assert_not_equal(ds.id, first_id)
+
+
+# While os.symlink does work on windows (since vista), os.path.realpath
+# doesn't resolve such symlinks. This has all kinds of implications.
+# Hopefully this can be dealt with, when we switch to using pathlib
+# (see datalad-revolution).
+@known_failure_windows
+@with_tempfile
+@with_tempfile
+@with_tempfile
+@with_tempfile(mkdir=True)
+@with_tempfile
+def test_symlinked_dataset_properties(repo1, repo2, repo3, non_repo, symlink):
+
+    ds = Dataset(repo1).create()
+
+    # now, let ds be a symlink and change that symlink to point to different
+    # things:
+    ar2 = AnnexRepo(repo2)
+    ar3 = AnnexRepo(repo3)
+    assert_true(os.path.isabs(non_repo))
+
+    os.symlink(repo1, symlink)
+    ds_link = Dataset(symlink)
+    assert_is(ds_link.repo, ds.repo)  # same Repo instance
+    assert_is_not(ds_link, ds)  # but not the same Dataset instance
+    assert_is(ds_link.config, ds.repo.config)
+    assert_true(ds_link._cfg_bound)
+    assert_is_not_none(ds_link.id)
+    # same id, although different Dataset instance:
+    assert_equal(ds_link.id, ds.id)
+
+    os.unlink(symlink)
+    os.symlink(repo2, symlink)
+
+    assert_is(ds_link.repo, ar2)  # same Repo instance
+    assert_is(ds_link.config, ar2.config)
+    assert_true(ds_link._cfg_bound)
+    # id is None again, since this repository is an annex but there was no
+    # Dataset.create() called yet.
+    assert_is_none(ds_link.id)
+
+    os.unlink(symlink)
+    os.symlink(repo3, symlink)
+
+    assert_is(ds_link.repo, ar3)  # same Repo instance
+    assert_is(ds_link.config, ar3.config)
+    assert_true(ds_link._cfg_bound)
+    # id is None again, since this repository is an annex but there was no
+    # Dataset.create() called yet.
+    assert_is_none(ds_link.id)
+
+    os.unlink(symlink)
+    os.symlink(non_repo, symlink)
+
+    assert_is_none(ds_link.repo)
+    assert_is_not(ds_link.config, ar3.config)
+    assert_false(ds_link._cfg_bound)
+    assert_is_none(ds_link.id)
