@@ -9,7 +9,6 @@
 
 """
 
-from datalad.tests.utils import known_failure_v6
 from datalad.tests.utils import known_failure_direct_mode
 
 
@@ -458,7 +457,6 @@ def test_remove_nowhining(path):
 
 
 @usecase
-@known_failure_v6  # https://github.com/datalad/datalad/pull/2391#issuecomment-379414293
 @skip_if_no_network
 @with_tempfile(mkdir=True)
 @use_cassette('test_remove_recursive_2')
