@@ -255,7 +255,7 @@ class BaseDownloader(object):
         Raises
         ------
         DownloadError
-          If either no known credentials type, or user refuses to update
+          If no known credentials type or user refuses to update
         """
         title = "{msg} access to {url} has failed.".format(
             msg=denied_msg, url=url)
@@ -284,7 +284,7 @@ class BaseDownloader(object):
         else:
             action_msg = "enter other credentials in case they were updated?"
 
-            if ui.yesno(
+            if self.credential and ui.yesno(
                     title=title,
                     text="Do you want to %s" % action_msg):
                 self.credential.enter_new()
@@ -498,9 +498,9 @@ class BaseDownloader(object):
         url: str
           URL to download
         cache: bool, optional
-          If None, config is consulted either results should be cached.
-          Cache is operating based on url, so no verification of any kind
-          is carried out
+          If None, config is consulted to determine whether results should be
+          cached. Cache is operating based on url, so no verification of any
+          kind is carried out
 
         Returns
         -------
