@@ -19,11 +19,11 @@ from ..consts import ARCHIVES_TEMP_DIR
 from ..consts import ANNEX_TEMP_DIR
 from ..consts import SEARCH_INDEX_DOTGITDIR
 
+from datalad.support.gitrepo import GitRepo
 from datalad.support.constraints import EnsureNone
 from datalad.distribution.dataset import EnsureDataset
 from datalad.distribution.dataset import require_dataset
 from datalad.distribution.dataset import datasetmethod
-from datalad.distribution.utils import get_git_dir
 from datalad.interface.annotate_paths import AnnotatePaths
 from datalad.interface.common_opts import recursion_flag
 from datalad.interface.common_opts import recursion_limit
@@ -90,7 +90,7 @@ class Clean(Interface):
                 yield ap
                 continue
             d = ap['path']
-            gitdir = get_git_dir(d)
+            gitdir = GitRepo.get_git_dir(d)
             for dirpath, flag, msg, sing_pl in [
                 (ARCHIVES_TEMP_DIR, "cached-archives",
                  "temporary archive", ("directory", "directories")),
