@@ -20,9 +20,10 @@ from os.path import join as opj
 
 from datalad.distribution.dataset import Dataset
 from datalad.interface.ls_webui import machinesize, ignored, fs_traverse, \
-    _ls_json
+    _ls_json, UNKNOWN_SIZE
 from datalad.support.annexrepo import AnnexRepo
 from datalad.support.gitrepo import GitRepo
+from datalad.tests.utils import known_failure_direct_mode
 from datalad.tests.utils import with_tree
 from datalad.utils import swallow_logs, swallow_outputs, _path_
 # needed below as bound dataset method
@@ -36,6 +37,7 @@ def test_machinesize():
     for key, value in {'Byte': 0, 'Bytes': 0, 'kB': 1, 'MB': 2, 'GB': 3, 'TB': 4, 'PB': 5}.items():
         assert_equal(1.0*(1000**value), machinesize('1 ' + key))
     assert_raises(ValueError, machinesize, 't byte')
+    assert_equal(0, machinesize(UNKNOWN_SIZE))
 
 
 @with_tree(
@@ -132,6 +134,7 @@ def test_fs_traverse(topdir):
             assert_equal(brokenlink['size']['total'], '3 Bytes')
 
 
+@known_failure_direct_mode  #FIXME
 @with_tree(
     tree={'dir': {'.fgit': {'ab.txt': '123'},
                   'subdir': {'file1.txt': '123',
@@ -191,6 +194,11 @@ def test_ls_json(topdir, topurl):
         with open(get_metapath(dspath, *path)) as f:
             return js.load(f)
 
+    # Let's see that there is no crash if one of the files is available only
+    # in relaxed URL mode, so no size could be picked up
+    ds.repo.add_url_to_file(
+        'fromweb', topurl + '/noteventhere', options=['--relaxed'])
+
     for all_ in [True, False]:  # recurse directories
         for recursive in [True, False]:
             for state in ['file', 'delete']:
@@ -264,3 +272,7 @@ def test_ls_json(topdir, topurl):
                         if item['name'] == ('subdsfile.txt' or 'subds')
                     ][0]
                     assert_equal(subds['size']['total'], '3 Bytes')
+
+                assert_equal(
+                    topds_nodes['fromweb']['size']['total'], UNKNOWN_SIZE
+                )
\ No newline at end of file
