@@ -1,4 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
+# -*- coding: utf-8 -*-
 # ex: set sts=4 ts=4 sw=4 noet:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
@@ -41,7 +42,7 @@ from datalad.tests.utils import assert_status
 from datalad.tests.utils import assert_result_count
 from datalad.tests.utils import assert_not_in
 from datalad.tests.utils import assert_result_values_equal
-from datalad.tests.utils import skip_v6
+from datalad.tests.utils import skip_v6_or_later
 from datalad.tests.utils import known_failure_windows
 
 
@@ -269,15 +270,15 @@ def test_recursive_save(path):
 @with_tempfile()
 def test_save_message_file(path):
     ds = Dataset(path).create()
-    with assert_raises(IncompleteResultsError):
+    with assert_raises(ValueError):
         ds.save("blah", message="me", message_file="and me")
 
     create_tree(path, {"foo": "x",
-                       "msg": "add foo"})
+                       "msg": u"add β"})
     ds.add("foo", save=False)
     ds.save(message_file=opj(ds.path, "msg"))
     assert_equal(ds.repo.format_commit("%s"),
-                 "add foo")
+                 u"add β")
 
 
 def test_renamed_file():
@@ -368,7 +369,7 @@ def test_symlinked_relpath(path):
         ds.repo.add(later, git=True)
         ds.save("committing", path=later)
 
-    skip_v6(method='pass')(ok_clean_git)(dspath)
+    ok_clean_git(dspath)
 
 
 # two subdatasets not possible in direct mode
@@ -437,14 +438,14 @@ def test_gh2043p1(path):
         save('.')  #  because the first arg is the dataset
     # state of the file (unlocked/locked) is committed as well, and the
     # test doesn't lock the file again
-    skip_v6(method='pass')(ok_clean_git)(ds.path, untracked=['2', '3'])
+    skip_v6_or_later(method='pass')(ok_clean_git)(ds.path, untracked=['2', '3'])
     with chpwd(path):
         # but when a path is given, anything that matches this path
         # untracked or not is added/saved
         save(path='.')
     # state of the file (unlocked/locked) is committed as well, and the
     # test doesn't lock the file again
-    skip_v6(method='pass')(ok_clean_git)(ds.path)
+    skip_v6_or_later(method='pass')(ok_clean_git)(ds.path)
 
 
 @with_tree({
