@@ -54,6 +54,16 @@ def check_api(no_annex, path):
         # precious file
         if extractor_ep.name == 'datalad_core':
             assert 'file.dat' in cm
+        elif extractor_ep.name == 'annex':
+            if not no_annex:
+                # verify correct key, which is the same for all files of 0 size
+                assert_equal(
+                    cm['file.dat']['key'],
+                    'MD5E-s0--d41d8cd98f00b204e9800998ecf8427e.dat'
+                )
+            else:
+                # no metadata on that file
+                assert not cm
         processed_extractors.append(extractor_ep.name)
     assert "datalad_core" in processed_extractors, \
         "Should have managed to find at least the core extractor extractor"
