@@ -15,18 +15,16 @@ import glob
 import logging
 import re
 import os
-from os.path import dirname
-from os.path import relpath
-from os.path import normpath
-from os.path import curdir
-from os.path import exists
-from os.path import lexists
-from os.path import join as opj
-from collections import OrderedDict
-from collections import Mapping
-from six import binary_type, string_types
-from six import iteritems
-
+import os.path as op
+from collections import (
+    OrderedDict,
+    Mapping,
+)
+from six import (
+    binary_type,
+    string_types,
+    iteritems,
+)
 from datalad import cfg
 from datalad.interface.annotate_paths import AnnotatePaths
 from datalad.interface.base import Interface
@@ -34,43 +32,54 @@ from datalad.interface.results import get_status_dict
 from datalad.interface.utils import eval_results
 from datalad.interface.base import build_doc
 from datalad.metadata.definitions import version as vocabulary_version
-from datalad.support.constraints import EnsureNone
-from datalad.support.constraints import EnsureBool
-from datalad.support.constraints import EnsureStr
+from datalad.support.constraints import (
+    EnsureNone,
+    EnsureBool,
+    EnsureStr,
+)
 from datalad.support.gitrepo import GitRepo
 from datalad.support.annexrepo import AnnexRepo
 from datalad.support.param import Parameter
 import datalad.support.ansi_colors as ac
-from datalad.support.json_py import load as jsonload
-from datalad.support.json_py import load_xzstream
-from datalad.interface.common_opts import recursion_flag
-from datalad.interface.common_opts import reporton_opt
-from datalad.distribution.dataset import Dataset
-from datalad.distribution.dataset import EnsureDataset
-from datalad.distribution.dataset import datasetmethod
-from datalad.distribution.dataset import require_dataset
-from datalad.utils import assure_list
-from datalad.utils import path_is_subpath, path_startswith
-from datalad.utils import as_unicode
+from datalad.support.json_py import (
+    load as jsonload,
+    load_xzstream,
+)
+from datalad.interface.common_opts import (
+    recursion_flag,
+    reporton_opt,
+)
+from datalad.distribution.dataset import (
+    Dataset,
+    EnsureDataset,
+    datasetmethod,
+    require_dataset,
+)
+from datalad.utils import (
+    assure_list,
+    path_is_subpath,
+    path_startswith,
+    as_unicode,
+)
 from datalad.ui import ui
 from datalad.dochelpers import exc_str
-from datalad.dochelpers import single_or_plural
-from datalad.consts import METADATA_DIR, METADATA_FILENAME
-
+from datalad.consts import (
+    METADATA_DIR,
+    METADATA_FILENAME,
+)
 from datalad.log import log_progress
+
 lgr = logging.getLogger('datalad.metadata.metadata')
 
 aggregate_layout_version = 1
-agginfo_relpath_template = opj(
-    '.datalad',
-    'metadata',
-    'aggregate_v{}.json')
-agginfo_relpath = agginfo_relpath_template.format(aggregate_layout_version)
 
 # relative paths which to exclude from any metadata processing
 # including anything underneath them
 exclude_from_metadata = ('.datalad', '.git', '.gitmodules', '.gitattributes')
 
+# TODO filepath_info is obsolete
+location_keys = ('dataset_info', 'content_info', 'filepath_info')
+
 
 def get_metadata_type(ds):
     """Return the metadata type(s)/scheme(s) of a dataset
@@ -97,19 +106,12 @@ def get_metadata_type(ds):
     return []
 
 
-class MetadataDict(dict):
-    """Metadata dict helper class"""
-    # TODO no longer needed ATM, but keeping for now, avoiding the
-    # big diff
-    pass
-
-
 def _load_json_object(fpath, cache=None):
     if cache is None:
         cache = {}
     obj = cache.get(
         fpath,
-        jsonload(fpath, fixup=True) if lexists(fpath) else {})
+        jsonload(fpath, fixup=True) if op.lexists(fpath) else {})
     cache[fpath] = obj
     return obj
 
@@ -121,7 +123,7 @@ def _load_xz_json_stream(fpath, cache=None):
         fpath,
         {s['path']: {k: v for k, v in iteritems(s) if k != 'path'}
          # take out the 'path' from the payload
-         for s in load_xzstream(fpath)} if lexists(fpath) else {})
+         for s in load_xzstream(fpath)} if op.lexists(fpath) else {})
     cache[fpath] = obj
     return obj
 
@@ -133,20 +135,25 @@ def _get_metadatarelevant_paths(ds, subds_relpaths):
 
 
 def _get_containingds_from_agginfo(info, rpath):
-    """Return the relative path of a dataset that contains a relative query path
+    """Return the path of a dataset that contains a query path
+
+    If a query path matches a dataset path directly, the matching dataset path
+    is return -- not the parent dataset!
 
     Parameters
     ----------
     info : dict
-      Content of aggregate.json (dict with relative subdataset paths as keys)
+      Content of aggregate.json (dict with (relative) subdataset paths as keys)
     rpath : str
-      Relative query path
+      Query path can be absolute or relative, but must match the convention
+      used in the info dict.
 
     Returns
     -------
     str or None
-      None is returned if the is no match, the relative path of the closest
-      containing subdataset otherwise.
+      None is returned if there is no match, the path of the closest
+      containing subdataset otherwise (in the convention used in the
+      info dict).
     """
     if rpath in info:
         dspath = rpath
@@ -197,30 +204,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
     """
     from datalad.coreapi import get
     # look for and load the aggregation info for the base dataset
-    info_fpath = opj(ds.path, agginfo_relpath)
-    agg_base_path = dirname(info_fpath)
-    agginfos = _load_json_object(info_fpath)
-    if not agginfos and not exists(info_fpath):
-        # This dataset does not have aggregated metadata.  Does it have any
-        # other version?
-        info_glob = agginfo_relpath_template.format('*')
-        info_files = glob.glob(info_glob)
-        msg = "Found no aggregated metadata info file %s." \
-              % info_fpath
-        old_metadata_file = opj(ds.path, METADATA_DIR, METADATA_FILENAME)
-        if exists(old_metadata_file):
-            msg += " Found metadata generated with pre-0.10 version of " \
-                   "DataLad, but it will not be used."
-        upgrade_msg = ""
-        if info_files:
-            msg += " Found following info files, which might have been " \
-                   "generated with newer version(s) of datalad: %s." \
-                   % (', '.join(info_files))
-            upgrade_msg = ", upgrade datalad"
-        msg += " You will likely need to either update the dataset from its " \
-               "original location,%s or reaggregate metadata locally." \
-               % upgrade_msg
-        lgr.warning(msg)
+    agginfos, agg_base_path = load_ds_aggregate_db(ds)
 
     # cache once loaded metadata objects for additional lookups
     # TODO possibly supply this cache from outside, if objects could
@@ -236,7 +220,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
     for ap in aps:
         # all metadata is registered via its relative path to the
         # dataset that is being queried
-        rpath = relpath(ap['path'], start=ds.path)
+        rpath = op.relpath(ap['path'], start=ds.path)
         if rpath in reported:
             # we already had this, probably via recursion of some kind
             continue
@@ -252,7 +236,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
             # could happen if there was no aggregated metadata at all
             # or the path is in this dataset, but luckily the queried dataset
             # is known to be present
-            containing_ds = curdir
+            containing_ds = op.curdir
         rap['metaprovider'] = containing_ds
 
         # build list of datasets and paths to be queried for this annotated path
@@ -265,7 +249,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
             matching_subds = [{'metaprovider': sub, 'rpath': sub, 'type': 'dataset'}
                               for sub in sorted(agginfos)
                               # we already have the base dataset
-                              if (rpath == curdir and sub != curdir) or
+                              if (rpath == op.curdir and sub != op.curdir) or
                               path_is_subpath(sub, rpath)]
             to_query.extend(matching_subds)
 
@@ -303,7 +287,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
             'Verifying/achieving local availability of %i metadata objects',
             len(objfiles))
         if objfiles:
-            get(path=[dict(path=opj(agg_base_path, of),
+            get(path=[dict(path=op.join(agg_base_path, of),
                            parentds=ds.path, type='file')
                       for of in objfiles if of],
                 dataset=ds,
@@ -321,7 +305,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
             dsobjloc = dsinfo.get('dataset_info', None)
             if dsobjloc is not None:
                 dsmeta = _load_json_object(
-                    opj(agg_base_path, dsobjloc),
+                    op.join(agg_base_path, dsobjloc),
                     cache=cache['objcache'])
 
             for r in _query_aggregated_metadata_singlepath(
@@ -334,7 +318,7 @@ def query_aggregated_metadata(reporton, ds, aps, recursive=False,
                 if 'query_matched' in ap:
                     r['query_matched'] = ap['query_matched']
                 if r.get('type', None) == 'file':
-                    r['parentds'] = normpath(opj(ds.path, qap['metaprovider']))
+                    r['parentds'] = op.normpath(op.join(ds.path, qap['metaprovider']))
                 yield r
                 reported.add(qap['rpath'])
 
@@ -347,7 +331,7 @@ def _query_aggregated_metadata_singlepath(
     rpath = qap['rpath']
     containing_ds = qap['metaprovider']
     qtype = qap.get('type', None)
-    if (rpath == curdir or rpath == containing_ds) and \
+    if (rpath == op.curdir or rpath == containing_ds) and \
             ((reporton is None and qtype == 'dataset') or \
              reporton in ('datasets', 'all')):
         # this is a direct match for a dataset (we only have agginfos for
@@ -356,7 +340,7 @@ def _query_aggregated_metadata_singlepath(
             status='ok',
             metadata=dsmeta,
             # normpath to avoid trailing dot
-            path=normpath(opj(ds.path, rpath)),
+            path=op.normpath(op.join(ds.path, rpath)),
             type='dataset')
         # all info on the dataset is gathered -> eject
         yield res
@@ -369,18 +353,18 @@ def _query_aggregated_metadata_singlepath(
     #
     # content info dicts have metadata stored under paths that are relative
     # to the dataset they were aggregated from
-    rparentpath = relpath(rpath, start=containing_ds)
+    rparentpath = op.relpath(rpath, start=containing_ds)
 
     # so we have some files to query, and we also have some content metadata
     contentmeta = _load_xz_json_stream(
-        opj(agg_base_path, contentinfo_objloc),
+        op.join(agg_base_path, contentinfo_objloc),
         cache=cache['objcache']) if contentinfo_objloc else {}
 
     for fpath in [f for f in contentmeta.keys()
-                  if rparentpath == curdir or
+                  if rparentpath == op.curdir or
                   path_startswith(f, rparentpath)]:
         # we might be onto something here, prepare result
-        metadata = MetadataDict(contentmeta.get(fpath, {}))
+        metadata = contentmeta.get(fpath, {})
 
         # we have to pull out the context for each extractor from the dataset
         # metadata
@@ -397,8 +381,8 @@ def _query_aggregated_metadata_singlepath(
         res = get_status_dict(
             status='ok',
             # the specific match within the containing dataset
-            # normpath() because containing_ds could be `curdir`
-            path=normpath(opj(ds.path, containing_ds, fpath)),
+            # normpath() because containing_ds could be `op.curdir`
+            path=op.normpath(op.join(ds.path, containing_ds, fpath)),
             # we can only match files
             type='file',
             metadata=metadata)
@@ -454,8 +438,7 @@ def _get_metadata(ds, types, global_meta=None, content_meta=None, paths=None):
     types : list
     """
     errored = False
-    dsmeta = MetadataDict()
-    # each item in here will be a MetadataDict, but not the whole thing
+    dsmeta = dict()
     contentmeta = {}
 
     if global_meta is not None and content_meta is not None and \
@@ -517,7 +500,7 @@ def _get_metadata(ds, types, global_meta=None, content_meta=None, paths=None):
                 'Failed %s metadata extraction from %s', mtype_key, ds,
             )
             raise ValueError(
-                'Enable metadata extractor %s is not available in this installation',
+                'Enabled metadata extractor %s is not available in this installation',
                 mtype_key)
         try:
             extractor_cls = extractors[mtype_key].load()
@@ -610,7 +593,6 @@ def _get_metadata(ds, types, global_meta=None, content_meta=None, paths=None):
             #elif not meta:
             #    continue
 
-            meta = MetadataDict(meta)
             # apply filters
             meta = _filter_metadata_fields(
                 meta,
@@ -777,6 +759,100 @@ class ReadOnlyDict(Mapping):
         return self._hash
 
 
+def get_ds_aggregate_db_locations(ds, version='default'):
+    """Returns the location of a dataset's aggregate metadata DB
+
+    Parameters
+    ----------
+    ds : Dataset
+      Dataset instance to query
+    version : str
+      DataLad aggregate metadata layout version. At the moment only a single
+      version exists. 'default' will return the locations for the current default
+      layout version.
+
+    Returns
+    -------
+    db_location, db_object_base_path
+      Absolute paths to the DB itself, and to the basepath to resolve relative
+      object references in the database. Either path may not exist in the
+      queried dataset.
+    """
+    layout_version = aggregate_layout_version \
+        if version == 'default' else version
+
+    agginfo_relpath_template = op.join(
+        '.datalad',
+        'metadata',
+        'aggregate_v{}.json')
+    agginfo_relpath = agginfo_relpath_template.format(layout_version)
+    info_fpath = op.join(ds.path, agginfo_relpath)
+    agg_base_path = op.dirname(info_fpath)
+    # not sure if this is the right place with these check, better move then to a higher level
+    if not op.exists(info_fpath):
+        if version == 'default':
+            # caller had no specific idea what metadata version is needed/available
+            # This dataset does not have aggregated metadata.  Does it have any
+            # other version?
+            info_glob = agginfo_relpath_template.format('*')
+            info_files = glob.glob(info_glob)
+            msg = "Found no aggregated metadata info file %s." \
+                  % info_fpath
+            old_metadata_file = op.join(ds.path, METADATA_DIR, METADATA_FILENAME)
+            if op.exists(old_metadata_file):
+                msg += " Found metadata generated with pre-0.10 version of " \
+                       "DataLad, but it will not be used."
+            upgrade_msg = ""
+            if info_files:
+                msg += " Found following info files, which might have been " \
+                       "generated with newer version(s) of datalad: %s." \
+                       % (', '.join(info_files))
+                upgrade_msg = ", upgrade datalad"
+            msg += " You will likely need to either update the dataset from its " \
+                   "original location%s or reaggregate metadata locally." \
+                   % upgrade_msg
+            lgr.warning(msg)
+    return info_fpath, agg_base_path
+
+
+def load_ds_aggregate_db(ds, version='default', abspath=False):
+    """Load a dataset's aggregate metadata database
+
+    Parameters
+    ----------
+    ds : Dataset
+      Dataset instance to query
+    version : str
+      DataLad aggregate metadata layout version. At the moment only a single
+      version exists. 'default' will return the content of the current default
+      aggregate database version.
+
+    Returns
+    -------
+    dict [, str]
+      A dictionary with the database content is return. If abspath is True,
+      all paths in the dictionary (datasets, metadata object archives) are
+      absolute. If abspath is False, all paths are relative, and the metadata
+      object base path is return as a second value.
+    """
+    info_fpath, agg_base_path = get_ds_aggregate_db_locations(ds, version)
+
+    # save to call even with a non-existing location
+    agginfos = _load_json_object(info_fpath)
+
+    if abspath:
+        return {
+            # paths in DB on disk are always relative
+            # make absolute to ease processing during aggregation
+            op.normpath(op.join(ds.path, p)):
+            {k: op.normpath(op.join(agg_base_path, v)) if k in location_keys else v
+             for k, v in props.items()}
+            for p, props in agginfos.items()
+        }
+    else:
+        return agginfos, agg_base_path
+
+
 @build_doc
 class Metadata(Interface):
     """Metadata reporting for files and entire datasets
@@ -871,8 +947,12 @@ class Metadata(Interface):
                 refds_path,
                 check_installed=True,
                 purpose='aggregate metadata query')
-            info_fpath = opj(ds.path, agginfo_relpath)
-            if not exists(info_fpath):
+            agginfos = load_ds_aggregate_db(
+                ds,
+                version=str(aggregate_layout_version),
+                abspath=True
+            )
+            if not agginfos:
                 # if there has ever been an aggregation run, this file would
                 # exist, hence there has not been and we need to tell this
                 # to people
@@ -883,11 +963,9 @@ class Metadata(Interface):
                     logger=lgr,
                     message='metadata aggregation has never been performed in this dataset')
                 return
-            agginfos = _load_json_object(info_fpath)
             parentds = []
-            for sd in sorted(agginfos):
-                info = agginfos[sd]
-                dspath = normpath(opj(ds.path, sd))
+            for dspath in sorted(agginfos):
+                info = agginfos[dspath]
                 if parentds and not path_is_subpath(dspath, parentds[-1]):
                     parentds.pop()
                 info.update(
@@ -895,7 +973,7 @@ class Metadata(Interface):
                     type='dataset',
                     status='ok',
                 )
-                if sd == curdir:
+                if dspath == ds.path:
                     info['layout_version'] = aggregate_layout_version
                 if parentds:
                     info['parentds'] = parentds[-1]
@@ -909,7 +987,7 @@ class Metadata(Interface):
         if not dataset and not path:
             # makes no sense to have no dataset, go with "here"
             # error generation happens during annotation
-            path = curdir
+            path = op.curdir
 
         content_by_ds = OrderedDict()
         for ap in AnnotatePaths.__call__(
@@ -983,7 +1061,7 @@ class Metadata(Interface):
             # logging complained about this already
             return
         # list the path, available metadata keys, and tags
-        path = relpath(res['path'],
+        path = op.relpath(res['path'],
                        res['refds']) if res.get('refds', None) else res['path']
         meta = res.get('metadata', {})
         ui.message('{path}{type}:{spacer}{meta}{tags}'.format(
