@@ -132,10 +132,10 @@ class NoAnnex(Interface):
                 return
 
         gitattr_file = opj(gitattr_dir, '.gitattributes')
-        with open(gitattr_file, 'a') as fp:
-            for p in pattern:
-                fp.write('{} annex.largefiles=nothing\n'.format(p))
-            yield dict(res_kwargs, status='ok')
+        dataset.repo.set_gitattributes(
+            [(p, {'annex.largefiles': 'nothing'}) for p in pattern],
+            attrfile=gitattr_file)
+        yield dict(res_kwargs, status='ok')
 
         for r in dataset.add(
                 gitattr_file,
