@@ -4,6 +4,7 @@ import os
 from contextlib import contextmanager
 
 from .path import exists
+from ..utils import unlink
 
 import logging
 lgr = logging.getLogger('datalad.locking')
@@ -97,6 +98,6 @@ def lock_if_check_fails(
     finally:
         if lock.acquired:
             lgr.debug("Releasing lock %s", lock_filename)
-            if exists(lock_filename):
-                os.unlink(lock_filename)
             lock.release()
+            if exists(lock_filename):
+                unlink(lock_filename)
