@@ -224,7 +224,7 @@ def retry_urlopen(url, retries=3):
 
 
 def is_url_quoted(url):
-    """Return either URL looks being already quoted
+    """Return whether URL looks being already quoted
     """
     try:
         url_ = urlunquote(url)
@@ -235,7 +235,7 @@ def is_url_quoted(url):
 
 
 def same_website(url_rec, u_rec):
-    """Decide either a link leads to external site
+    """Decide whether a link leads to external site
 
     Parameters
     ----------
