@@ -14,6 +14,7 @@ from datalad.tests.utils import known_failure_v6
 
 import logging
 from functools import partial
+from glob import glob
 import os
 from os import mkdir
 from os.path import join as opj
@@ -37,6 +38,7 @@ import gc
 from datalad.cmd import Runner
 
 from datalad.support.external_versions import external_versions
+from datalad.support import path as op
 
 from datalad.support.sshconnector import get_connection_hash
 
@@ -69,6 +71,7 @@ from datalad.tests.utils import ok_
 from datalad.tests.utils import ok_git_config_not_empty
 from datalad.tests.utils import ok_annex_get
 from datalad.tests.utils import ok_clean_git
+from datalad.tests.utils import ok_file_under_git
 from datalad.tests.utils import ok_file_has_content
 from datalad.tests.utils import swallow_logs
 from datalad.tests.utils import swallow_outputs
@@ -79,6 +82,7 @@ from datalad.tests.utils import OBSCURE_FILENAME
 from datalad.tests.utils import SkipTest
 from datalad.tests.utils import skip_ssh
 from datalad.tests.utils import find_files
+from datalad.tests.utils import slow
 
 from datalad.support.exceptions import CommandError
 from datalad.support.exceptions import CommandNotAvailableError
@@ -201,7 +205,7 @@ def test_AnnexRepo_set_direct_mode(src, dst):
 
     ar = AnnexRepo.clone(src, dst)
 
-    if ar.config.getint("annex", "version") >= 6:
+    if ar.supports_unlocked_pointers:
         # there's no direct mode available:
         assert_raises(CommandError, ar.set_direct_mode, True)
         raise SkipTest("Test not applicable in repository version >= 6")
@@ -223,7 +227,7 @@ def test_AnnexRepo_set_direct_mode(src, dst):
 @with_tempfile
 def test_AnnexRepo_annex_proxy(src, annex_path):
     ar = AnnexRepo.clone(src, annex_path)
-    if ar.config.getint("annex", "version") >= 6:
+    if ar.supports_unlocked_pointers:
         # there's no direct mode available and therefore no 'annex proxy':
         assert_raises(CommandError, ar.proxy, ['git', 'status'])
         raise SkipTest("Test not applicable in repository version >= 6")
@@ -246,15 +250,16 @@ def test_AnnexRepo_get_file_key(src, annex_path):
     ar = AnnexRepo.clone(src, annex_path)
 
     # test-annex.dat should return the correct key:
-    eq_(
-        ar.get_file_key("test-annex.dat"),
-        'SHA256E-s28--2795fb26981c5a687b9bf44930cc220029223f472cea0f0b17274f4473181e7b.dat')
+    test_annex_key = \
+        'SHA256E-s28' \
+        '--2795fb26981c5a687b9bf44930cc220029223f472cea0f0b17274f4473181e7b.dat'
+    eq_(ar.get_file_key("test-annex.dat"), test_annex_key)
 
     # and should take a list with an empty string as result, if a file wasn't
     # in annex:
     eq_(
         ar.get_file_key(["filenotpresent.wtf", "test-annex.dat"]),
-        ['', 'SHA256E-s28--2795fb26981c5a687b9bf44930cc220029223f472cea0f0b17274f4473181e7b.dat']
+        ['', test_annex_key]
     )
 
     # test.dat is actually in git
@@ -266,6 +271,12 @@ def test_AnnexRepo_get_file_key(src, annex_path):
     # filenotpresent.wtf doesn't even exist
     assert_raises(IOError, ar.get_file_key, "filenotpresent.wtf")
 
+    # if we force batch mode, no failure for not present or not annexed files
+    eq_(ar.get_file_key("filenotpresent.wtf", batch=True), '')
+    eq_(ar.get_file_key("test.dat", batch=True), '')
+    eq_(ar.get_file_key("test-annex.dat", batch=True), test_annex_key)
+
+
 
 @with_tempfile(mkdir=True)
 def test_AnnexRepo_get_outofspace(annex_path):
@@ -322,7 +333,7 @@ def test_AnnexRepo_file_has_content(batch, direct, src, annex_path):
     if not direct:  # There's no unlock in direct mode.
         ar.unlock(["test-annex.dat"])
         eq_(ar.file_has_content(["test-annex.dat"], batch=batch),
-            [ar.config.get("annex.version") == "6"])
+            [ar.supports_unlocked_pointers])
         with open(opj(annex_path, "test-annex.dat"), "a") as ofh:
             ofh.write("more")
         eq_(ar.file_has_content(["test-annex.dat"], batch=batch),
@@ -358,7 +369,7 @@ def test_AnnexRepo_is_under_annex(batch, direct, src, annex_path):
     if not direct:  # There's no unlock in direct mode.
         ar.unlock(["test-annex.dat"])
         eq_(ar.is_under_annex(["test-annex.dat"], batch=batch),
-            [ar.config.get("annex.version") == "6"])
+            [ar.supports_unlocked_pointers])
         with open(opj(annex_path, "test-annex.dat"), "a") as ofh:
             ofh.write("more")
         eq_(ar.is_under_annex(["test-annex.dat"], batch=batch),
@@ -900,11 +911,11 @@ def test_AnnexRepo_add_unexpected_direct_mode(path):
     # superproject is not.
     # There is no point in this test, if direct mode was enforced in the
     # superproject already (either by test run configuration or FS) or if the
-    # repositories are in V6 by default (where there is no direct mode)
+    # repositories are in V6+ by default (where there is no direct mode)
 
     top = AnnexRepo(path)
 
-    if top.is_direct_mode() or top.config.get("annex.version") == '6':
+    if top.is_direct_mode() or top.supports_unlocked_pointers:
         raise SkipTest("Nothing to test for")
 
     top.update_submodule('subm 1', init=True)
@@ -1217,7 +1228,7 @@ def test_annex_ssh(repo_path, remote_1_path, remote_2_path):
     # but socket was not touched:
     if localhost_was_open:
         # FIXME: occasionally(?) fails in V6:
-        if not ar.config.getint("annex", "version") == 6:
+        if not ar.supports_unlocked_pointers:
             ok_(exists(socket_2))
     else:
         ok_(not exists(socket_2))
@@ -1279,13 +1290,16 @@ def test_repo_version(path1, path2, path3):
     annex = AnnexRepo(path1, create=True, version=6)
     ok_clean_git(path1, annex=True)
     version = annex.repo.config_reader().get_value('annex', 'version')
-    eq_(version, 6)
+    # TODO: Since git-annex 7.20181031, v6 repos upgrade to v7. Once that
+    # version or later is our minimum required version, update this test and
+    # the one below to eq_(version, 7).
+    assert_in(version, [6, 7])
 
     # default from config item (via env var):
     with patch.dict('os.environ', {'DATALAD_REPO_VERSION': '6'}):
         annex = AnnexRepo(path2, create=True)
         version = annex.repo.config_reader().get_value('annex', 'version')
-        eq_(version, 6)
+        assert_in(version, [6, 7])
 
         # parameter `version` still has priority over default config:
         annex = AnnexRepo(path3, create=True, version=5)
@@ -1318,14 +1332,17 @@ def test_annex_copy_to(origin, clone):
     def ok_copy(command, **kwargs):
         # Check that we do pass to annex call only the list of files which we
         #  asked to be copied
-        assert_in('copied1', kwargs['annex_options'])
-        assert_in('copied2', kwargs['annex_options'])
-        assert_in('existed', kwargs['annex_options'])
+        assert_in('copied1', kwargs['files'])
+        assert_in('copied2', kwargs['files'])
+        assert_in('existed', kwargs['files'])
         return """
 {"command":"copy","note":"to target ...", "success":true, "key":"akey1", "file":"copied1"}
 {"command":"copy","note":"to target ...", "success":true, "key":"akey2", "file":"copied2"}
 {"command":"copy","note":"checking target ...", "success":true, "key":"akey3", "file":"existed"}
 """, ""
+    # Note that we patch _run_annex_command, which is also invoked by _run_annex_command_json
+    # which is in turn invoked first by copy_to for "find" operation.
+    # TODO: provide a dedicated handling within above ok_copy for 'find' command
     with patch.object(repo, '_run_annex_command', ok_copy):
         eq_(repo.copy_to(["copied2", "copied1", "existed"], "target"),
             ["copied1", "copied2"])
@@ -1491,6 +1508,7 @@ def test_is_available(batch, direct, p):
     # remove url
     urls = annex.get_urls(fname) #, **bkw)
     assert(len(urls) == 1)
+    eq_(urls, annex.get_urls(annex.get_file_key(fname), key=True))
     annex.rm_url(fname, urls[0])
 
     assert is_available(key, key=True) is False
@@ -1498,6 +1516,14 @@ def test_is_available(batch, direct, p):
     assert is_available(fname, remote='web') is False
 
 
+@with_tempfile(mkdir=True)
+def test_get_urls_none(path):
+    ar = AnnexRepo(path, create=True)
+    with open(opj(ar.path, "afile"), "w") as f:
+        f.write("content")
+    eq_(ar.get_urls("afile"), [])
+
+
 @with_tempfile(mkdir=True)
 def test_annex_add_no_dotfiles(path):
     ar = AnnexRepo(path, create=True)
@@ -1784,7 +1810,7 @@ def test_AnnexRepo_dirty(path):
     # modify to be the same
     with open(opj(path, 'file1.txt'), 'w') as f:
         f.write('whatever')
-    if not repo.config.getint("annex", "version") == 6:
+    if not repo.supports_unlocked_pointers:
         ok_(not repo.dirty)
     # modified file
     with open(opj(path, 'file1.txt'), 'w') as f:
@@ -1821,7 +1847,7 @@ def _test_status(ar):
 
     def sync_wrapper(push=False, pull=False, commit=False):
         # wraps common annex-sync call, since it currently fails under
-        # mysterious circumstances in V6 adjusted branch setups
+        # mysterious circumstances in V6+ adjusted branch setups
         try:
             ar.sync(push=push, pull=pull, commit=commit)
         except CommandError as e:
@@ -1846,7 +1872,7 @@ def _test_status(ar):
                 # But it almost works - so apperently nothing to do
                 import logging
                 lgr = logging.getLogger("datalad.support.tests.test-status")
-                lgr.warning("DEBUG: v6 sync failure")
+                lgr.warning("DEBUG: v6+ sync failure")
 
     stat = {'untracked': [],
             'deleted': [],
@@ -2061,7 +2087,7 @@ def _test_status(ar):
     ar.add('fifth')
     sync_wrapper()
 
-    if ar.config.getint("annex", "version") == 6:
+    if ar.supports_unlocked_pointers:
         # mixed annexed/not-annexed files ATm can't be committed with explicitly
         # given paths in v6
         # See:
@@ -2115,7 +2141,7 @@ def test_AnnexRepo_status(path, path2):
 
     ar = AnnexRepo(path, create=True)
     _test_status(ar)
-    if ar.config.getint("annex", "version") == 6:
+    if ar.supports_unlocked_pointers:
         # in case of v6 have a second run with adjusted branch feature:
         ar2 = AnnexRepo(path2, create=True)
         ar2.commit(msg="empty commit to create branch 'master'",
@@ -2291,7 +2317,7 @@ def test_AnnexRepo_get_corresponding_branch(path):
     eq_('master', ar.get_corresponding_branch())
 
     # special case v6 adjusted branch is not provided by a dedicated build:
-    if ar.config.getint("annex", "version") == 6:
+    if ar.supports_unlocked_pointers:
         ar.adjust()
         # as above, we still want to get 'master', while being on
         # 'adjusted/master(unlocked)'
@@ -2317,11 +2343,11 @@ def test_AnnexRepo_is_managed_branch(path):
     if ar.is_direct_mode():
         ok_(ar.is_managed_branch())
     else:
-        # ATM only direct mode and v6 adjusted branches should return True.
+        # ATM only direct mode and v6+ adjusted branches should return True.
         # Adjusted branch requires a call of git-annex-adjust and shouldn't
         # be the state of a fresh clone
         ok_(not ar.is_managed_branch())
-    if ar.config.getint("annex", "version") == 6:
+    if ar.supports_unlocked_pointers:
         ar.adjust()
         ok_(ar.is_managed_branch())
 
@@ -2398,3 +2424,135 @@ def test_error_reporting(path):
             'note': 'not found',
             'success': False}]
     )
+
+
+# http://git-annex.branchable.com/bugs/cannot_commit___34__annex_add__34__ed_modified_file_which_switched_its_largefile_status_to_be_committed_to_git_now/#comment-bf70dd0071de1bfdae9fd4f736fd1ec
+# https://github.com/datalad/datalad/issues/1651
+@with_tree(tree={
+    '.gitattributes': "** annex.largefiles=(largerthan=4b)",
+    'alwaysbig': 'a'*10,
+    'willnotgetshort': 'b'*10,
+    'tobechanged-git': 'a',
+    'tobechanged-annex': 'a'*10,
+})
+def check_commit_annex_commit_changed(unlock, path):
+    # Here we test commit working correctly if file was just removed
+    # (not unlocked), edited and committed back
+
+    # TODO: an additional possible interaction to check/solidify - if files
+    # first get unannexed (after being optionally unlocked first)
+    unannex = False
+
+    ar = AnnexRepo(path, create=True)
+    ar.add('.gitattributes')
+    ar.add('.')
+    ar.commit("initial commit")
+    ok_clean_git(path)
+    # Now let's change all but commit only some
+    files = [op.basename(p) for p in glob(op.join(path, '*'))]
+    if unlock:
+        ar.unlock(files)
+    if unannex:
+        ar.unannex(files)
+    create_tree(
+        path
+        , {
+            'alwaysbig': 'a'*11,
+            'willnotgetshort': 'b',
+            'tobechanged-git': 'aa',
+            'tobechanged-annex': 'a'*11,
+            'untracked': 'unique'
+        }
+        , remove_existing=True
+    )
+    ok_clean_git(
+        path
+        , index_modified=files if not unannex else ['tobechanged-git']
+        , untracked=['untracked'] if not unannex else
+          # all but the one in git now
+          ['alwaysbig', 'tobechanged-annex', 'untracked', 'willnotgetshort']
+    )
+
+    ar.commit("message", files=['alwaysbig', 'willnotgetshort'])
+    ok_clean_git(
+        path
+        , index_modified=['tobechanged-git', 'tobechanged-annex']
+        , untracked=['untracked']
+    )
+    ok_file_under_git(path, 'alwaysbig', annexed=True)
+    # This one is actually "questionable" since might be "correct" either way
+    # but it would be nice to have it at least consistent
+    ok_file_under_git(path, 'willnotgetshort', annexed=True)
+
+    ar.commit("message2", options=['-a']) # commit all changed
+    ok_clean_git(
+        path
+        , untracked=['untracked']
+    )
+    ok_file_under_git(path, 'tobechanged-git', annexed=False)
+    # TODO: direct mode gotcha!!!
+    ok_file_under_git(path, 'tobechanged-annex', annexed=not ar.is_direct_mode())
+
+
+def test_commit_annex_commit_changed():
+    for unlock in True, False:
+        yield check_commit_annex_commit_changed, unlock
+
+
+@with_tempfile(mkdir=True)
+def check_files_split_exc(cls, topdir):
+    from glob import glob
+    r = cls(topdir)
+    # absent files -- should not crash with "too long" but some other more
+    # meaningful exception
+    files = ["f" * 100 + "%04d" % f for f in range(100000)]
+    if isinstance(r, AnnexRepo):
+        # Annex'es add first checks for what is being added and does not fail
+        # for non existing files either ATM :-/  TODO: make consistent etc
+        r.add(files)
+    else:
+        with assert_raises(Exception) as ecm:
+            r.add(files)
+        assert_not_in('too long', str(ecm.exception))
+        assert_not_in('too many', str(ecm.exception))
+
+
+def test_files_split_exc():
+    for cls in GitRepo, AnnexRepo:
+        yield check_files_split_exc, cls
+
+
+_HEAVY_TREE = {
+    # might already run into 'filename too long' on windows probably
+    "d" * 98 + '%03d' % d: {
+        'f' * 98 + '%03d' % f: ''
+        for f in range(100)
+    }
+    for d in range(100)
+}
+
+
+@with_tree(tree=_HEAVY_TREE)
+def check_files_split(cls, topdir):
+    from glob import glob
+    r = cls(topdir)
+    dirs = glob(op.join(topdir, '*'))
+    files = glob(op.join(topdir, '*', '*'))
+
+    r.add(files)
+    r.commit(files=files)
+
+    # Let's modify and do dl.add for even a heavier test
+    # Now do for real on some heavy directory
+    import datalad.api as dl
+    for f in files:
+        os.unlink(f)
+        with open(f, 'w') as f:
+            f.write('1')
+    dl.add(dirs)
+
+
+@slow  # 313s  well -- if errors out - only 3 sec
+def test_files_split():
+    for cls in GitRepo, AnnexRepo:
+        yield check_files_split, cls
