@@ -246,8 +246,8 @@ def test_url_samples():
     _check_ri("file:///path/sp1", URL, localpath='/path/sp1', scheme='file', path='/path/sp1')
     # we don't do any magical comprehension for home paths/drives for windows
     # of file:// urls, thus leaving /~ and /c: for now:
-    _check_ri("file:///~/path/sp1", URL, localpath='/~/path/sp1', scheme='file', path='/~/path/sp1', exact_str=False)
-    _check_ri("file:///%7E/path/sp1", URL, localpath='/~/path/sp1', scheme='file', path='/~/path/sp1')
+    _check_ri("file:///~/path/sp1", URL, localpath='/~/path/sp1', scheme='file', path='/~/path/sp1')
+    _check_ri("file:///%7E/path/sp1", URL, localpath='/~/path/sp1', scheme='file', path='/~/path/sp1', exact_str=False)
     # not sure but let's check
     _check_ri("file:///c:/path/sp1", URL, localpath='/c:/path/sp1', scheme='file', path='/c:/path/sp1', exact_str=False)
 
@@ -390,6 +390,8 @@ def test_is_url():
 
 # TODO: RF with test_is_url to avoid duplication
 def test_is_datalad_compat_ri():
+    ok_(is_datalad_compat_ri('ssh://user:passw@host/path'))
+    ok_(is_datalad_compat_ri('http://example.com'))
     ok_(is_datalad_compat_ri('file://localhost/some'))
     ok_(is_datalad_compat_ri('///localhost/some'))
     nok_(is_datalad_compat_ri('relative'))
@@ -397,10 +399,11 @@ def test_is_datalad_compat_ri():
     nok_(is_datalad_compat_ri(123))
 
 
+@skip_if_on_windows
 def test_get_local_file_url_linux():
     eq_(get_local_file_url('/a'), 'file:///a')
     eq_(get_local_file_url('/a/b/c'), 'file:///a/b/c')
-    eq_(get_local_file_url('/a~'), 'file:///a%7E')
+    eq_(get_local_file_url('/a~'), 'file:///a~')
     eq_(get_local_file_url('/a b/'), 'file:///a%20b/')
 
 
