@@ -22,9 +22,10 @@ from datalad.tests.utils import with_tempfile
 @skip_ssh
 def test_exit_code():
     # will relay actual exit code on CommandError
-    cmd = ['sshrun', 'localhost', 'exit 42']
+    cmd = ['datalad', 'sshrun', 'localhost', 'exit 42']
     with assert_raises(SystemExit) as cme:
-        if isinstance(sys.stdout, StringIO):  # running nosetests without -s
+        # running nosetests without -s
+        if isinstance(sys.stdout, StringIO):  # pragma: no cover
             with swallow_outputs():  # need to give smth with .fileno ;)
                 main(cmd)
         else:
