@@ -97,7 +97,8 @@ def check_compress_file(ext, path, name):
 def test_compress_file():
     yield check_compress_file, '.tar.gz'
     yield check_compress_file, '.tar'
-    # yield check_compress_file, '.zip'
+    yield check_compress_file, '.zip'
+
 
 @with_tree(**tree_simplearchive)
 def test_ExtractedArchive(path):
