@@ -9,28 +9,39 @@
 """Test command call wrapper
 """
 
-from mock import patch
 import os
-from os.path import dirname, join as opj
+import os.path as op
 import sys
 import logging
 import shlex
 
-from .utils import ok_, eq_, assert_is, assert_equal, assert_false, \
-    assert_true, assert_greater, assert_raises, assert_in, SkipTest, unlink
+from .utils import (
+    ok_,
+    eq_,
+    assert_is,
+    assert_equal,
+    assert_false,
+    assert_true,
+    assert_greater,
+    assert_raises,
+    assert_in,
+    SkipTest,
+    skip_if_on_windows,
+    with_tempfile,
+    assert_cwd_unchanged,
+    ignore_nose_capturing_stdout,
+    swallow_outputs,
+    swallow_logs,
+    on_windows,
+    lgr,
+)
 
-from ..cmd import Runner, link_file_load
-from ..cmd import GitRunner
+from ..cmd import (
+    Runner,
+    GitRunner,
+)
 from ..support.exceptions import CommandError
 from ..support.protocol import DryRunProtocol
-from .utils import with_tempfile, assert_cwd_unchanged, \
-    ignore_nose_capturing_stdout, swallow_outputs, swallow_logs, \
-    on_linux, on_osx, on_windows, with_testrepos
-from .utils import lgr
-from ..utils import assure_unicode
-
-from .utils import local_testrepo_flavors
-from datalad.tests.utils import skip_if_on_windows
 
 
 @ignore_nose_capturing_stdout
@@ -181,7 +192,7 @@ def check_runner_heavy_output(log_online):
     # stucked yet.
 
     runner = Runner()
-    cmd = '%s %s' % (sys.executable, opj(dirname(__file__), "heavyoutput.py"))
+    cmd = '%s %s' % (sys.executable, op.join(op.dirname(__file__), "heavyoutput.py"))
 
     with swallow_outputs() as cm, swallow_logs():
         ret = runner.run(cmd,
@@ -222,64 +233,12 @@ def check_runner_heavy_output(log_online):
         assert not ret[0], "all messages went into `logged`"
 
 
-@skip_if_on_windows  # much too slow to finish in any reaosnable time on windows
 def test_runner_heavy_output():
+    skip_if_on_windows()
     for log_online in [False, True]:
         yield check_runner_heavy_output, log_online
 
 
-@with_tempfile
-def test_link_file_load(tempfile):
-    tempfile2 = tempfile + '_'
-
-    with open(tempfile, 'w') as f:
-        f.write("LOAD")
-
-    link_file_load(tempfile, tempfile2)  # this should work in general
-
-    ok_(os.path.exists(tempfile2))
-
-    with open(tempfile2, 'r') as f:
-        assert_equal(f.read(), "LOAD")
-
-    def inode(fname):
-        with open(fname) as fd:
-            return os.fstat(fd.fileno()).st_ino
-
-    def stats(fname, times=True):
-        """Return stats on the file which should have been preserved"""
-        with open(fname) as fd:
-            st = os.fstat(fd.fileno())
-            stats = (st.st_mode, st.st_uid, st.st_gid, st.st_size)
-            if times:
-                return stats + (st.st_atime, st.st_mtime)
-            else:
-                return stats
-            # despite copystat mtime is not copied. TODO
-            #        st.st_mtime)
-
-    if on_linux or on_osx:
-        # above call should result in the hardlink
-        assert_equal(inode(tempfile), inode(tempfile2))
-        assert_equal(stats(tempfile), stats(tempfile2))
-
-        # and if we mock absence of .link
-        def raise_AttributeError(*args):
-            raise AttributeError("TEST")
-
-        with patch('os.link', raise_AttributeError):
-            with swallow_logs(logging.WARNING) as cm:
-                link_file_load(tempfile, tempfile2)  # should still work
-                ok_("failed (TEST), copying file" in cm.out)
-
-    # should be a copy (either originally for windows, or after mocked call)
-    ok_(inode(tempfile) != inode(tempfile2))
-    with open(tempfile2, 'r') as f:
-        assert_equal(f.read(), "LOAD")
-    assert_equal(stats(tempfile, times=False), stats(tempfile2, times=False))
-    unlink(tempfile2)  # TODO: next two with_tempfile
-
-
 @with_tempfile(mkdir=True)
 def test_runner_failure(dir_):
     from ..support.annexrepo import AnnexRepo
@@ -294,6 +253,14 @@ def test_runner_failure(dir_):
     assert_equal(1, cme.exception.code)
 
 
+@with_tempfile(mkdir=True)
+def test_runner_failure_unicode(path):
+    # Avoid OBSCURE_FILENAME in hopes of windows-compatibility (gh-2929).
+    runner = Runner()
+    with assert_raises(CommandError), swallow_logs():
+        runner.run(u"β-command-doesnt-exist", cwd=path)
+
+
 @with_tempfile(mkdir=True)
 def test_git_path(dir_):
     from ..support.gitrepo import GitRepo
@@ -305,10 +272,10 @@ def test_git_path(dir_):
 @with_tempfile(mkdir=True)
 def test_runner_stdin(path):
     runner = Runner()
-    with open(opj(path, "test_input.txt"), "w") as f:
+    with open(op.join(path, "test_input.txt"), "w") as f:
         f.write("whatever")
 
-    with swallow_outputs() as cmo, open(opj(path, "test_input.txt"), "r") as fake_input:
+    with swallow_outputs() as cmo, open(op.join(path, "test_input.txt"), "r") as fake_input:
         runner.run(['cat'], log_stdout=False, stdin=fake_input)
         assert_in("whatever", cmo.out)
 
