@@ -29,6 +29,7 @@ from datalad.config import ConfigManager
 from datalad.cmd import CommandError
 
 from datalad.tests.utils import with_testsui
+from datalad.support.external_versions import external_versions
 
 # XXX tabs are intentional (part of the format)!
 # XXX put back! confuses pep8
@@ -64,16 +65,17 @@ def test_something(path, new_home):
 
     assert_true(cfg.has_section('something'))
     assert_false(cfg.has_section('somethingelse'))
-    assert_equal(sorted(cfg.sections()), ['onemore.complicated の beast with.dot', 'something'])
+    assert_equal(sorted(cfg.sections()),
+                 [u'onemore.complicated の beast with.dot', 'something'])
     assert_true(cfg.has_option('something', 'user'))
     assert_false(cfg.has_option('something', 'us?er'))
     assert_false(cfg.has_option('some?thing', 'user'))
     assert_equal(sorted(cfg.options('something')), ['myint', 'user'])
-    assert_equal(cfg.options('onemore.complicated の beast with.dot'), ['findme'])
+    assert_equal(cfg.options(u'onemore.complicated の beast with.dot'), ['findme'])
 
     assert_equal(
         sorted(cfg.items()),
-        [('onemore.complicated の beast with.dot.findme', '5.0'),
+        [(u'onemore.complicated の beast with.dot.findme', '5.0'),
          ('something.myint', '3'),
          ('something.user', ('name=Jane Doe', 'email=jd@example.com'))])
     assert_equal(
@@ -86,7 +88,7 @@ def test_something(path, new_home):
         cfg.get('something.user'),
         ('name=Jane Doe', 'email=jd@example.com'))
     assert_raises(KeyError, cfg.__getitem__, 'somedthing.user')
-    assert_equal(cfg.getfloat('onemore.complicated の beast with.dot', 'findme'), 5.0)
+    assert_equal(cfg.getfloat(u'onemore.complicated の beast with.dot', 'findme'), 5.0)
     assert_equal(cfg.getint('something', 'myint'), 3)
     assert_equal(cfg.getbool('something', 'myint'), True)
     assert_equal(cfg.getbool('doesnot', 'exist', default=True), True)
@@ -100,7 +102,7 @@ def test_something(path, new_home):
 
     # modification follows
     cfg.add('something.new', 'の')
-    assert_equal(cfg.get('something.new'), 'の')
+    assert_equal(cfg.get('something.new'), u'の')
     # sections are added on demand
     cfg.add('unheard.of', 'fame')
     assert_true(cfg.has_section('unheard.of'))
@@ -173,10 +175,11 @@ def test_something(path, new_home):
         # but after we unset the only value -- that section is no longer listed
         assert (not globalcfg.has_section('datalad.unittest'))
         assert_not_in('datalad.unittest.youcan', globalcfg)
-        # although it does leaves empty section behind in the file
-        ok_file_has_content(global_gitconfig, '[datalad "unittest"]', strip=True)
-        # remove_section to clean it up entirely
-        globalcfg.remove_section('datalad.unittest', where='global')
+        if external_versions['cmd:git'] < '2.18':
+            # older versions leave empty section behind in the file
+            ok_file_has_content(global_gitconfig, '[datalad "unittest"]', strip=True)
+            # remove_section to clean it up entirely
+            globalcfg.remove_section('datalad.unittest', where='global')
         ok_file_has_content(global_gitconfig, "")
 
     cfg = ConfigManager(
