@@ -11,11 +11,20 @@
 __docformat__ = 'restructuredtext'
 
 from .. import _UI_Switcher
-from ..dialog import DialogUI, ConsoleLog
-from ...tests.utils import assert_equal, assert_not_equal
-from ...tests.utils import assert_raises
-from ...tests.utils import assert_false
-from ...tests.utils import with_testsui
+from ..dialog import (
+    ConsoleLog,
+    DialogUI,
+    IPythonUI,
+)
+from ...tests.utils import (
+    assert_equal, assert_not_equal,
+    assert_raises,
+    assert_false,
+    ok_,
+    with_testsui,
+)
+
+from mock import patch
 
 
 def test_ui_switcher():
@@ -33,6 +42,16 @@ def test_ui_switcher():
 
     ui.set_backend('annex')
 
+    # Let's pretend we are under IPython
+    class ZMQInteractiveShell(object):
+        pass
+
+    with patch('datalad.utils.get_ipython',
+               lambda: ZMQInteractiveShell(),
+               create=True):
+        ui = _UI_Switcher()
+        assert (isinstance(ui.ui, IPythonUI))
+
 
 def test_tests_ui():
     ui = _UI_Switcher('dialog')
@@ -85,3 +104,6 @@ def test_with_testsui():
     @with_testsui(responses='a')
     def ask():
         assert_equal(ui.question('what is a?'), 'a')
+
+
+    ask()
\ No newline at end of file
