@@ -7,6 +7,7 @@
 
 import argparse
 import datetime
+import re
 
 
 class ManPageFormatter(argparse.HelpFormatter):
@@ -16,15 +17,19 @@ class ManPageFormatter(argparse.HelpFormatter):
     def __init__(self,
                  prog,
                  indent_increment=2,
-                 max_help_position=24,
-                 width=None,
+                 max_help_position=4,
+                 width=1000000,
                  section=1,
                  ext_sections=None,
                  authors=None,
                  version=None
                  ):
 
-        super(ManPageFormatter, self).__init__(prog)
+        super(ManPageFormatter, self).__init__(
+            prog,
+            indent_increment=indent_increment,
+            max_help_position=max_help_position,
+            width=width)
 
         self._prog = prog
         self._section = 1
@@ -53,30 +58,48 @@ class ManPageFormatter(argparse.HelpFormatter):
                        parser._mutually_exclusive_groups, prefix='')
         usage = self._format_usage(None, parser._actions,
                                    parser._mutually_exclusive_groups, '')
+        # replace too long list of commands with a single placeholder
+        usage = re.sub(r'{[^]]*?create,.*?}', ' COMMAND ', usage, flags=re.MULTILINE)
+        # take care of proper wrapping
+        usage = re.sub(r'\[([-a-zA-Z0-9]*)\s([a-zA-Z0-9{}|_]*)\]', r'[\1\~\2]', usage)
 
         usage = usage.replace('%s ' % self._prog, '')
-        usage = '.SH SYNOPSIS\n \\fB%s\\fR %s\n' % (self._markup(self._prog),
+        usage = '.SH SYNOPSIS\n.nh\n.HP\n\\fB%s\\fR %s\n.hy\n' % (self._markup(self._prog),
                                                     usage)
         return usage
 
     def _mk_title(self, prog):
         name_version = "\"{0} {1}\"".format(prog, self._version)
-        return '.TH {0} {1} {2} {3}\n'.format(prog, self._section,
-                                              self._today, name_version)
+        return '.TH "{0}" "{1}" "{2}" "{3}"\n'.format(
+            prog, self._section, self._today, name_version)
 
-    def _make_name(self, parser):
+    def _mk_name(self, prog, desc):
         """
         this method is in consitent with others ... it relies on
         distribution
         """
-        return '.SH NAME\n%s \\- %s\n' % (parser.prog,
-                                          parser.description)
+        desc = desc.splitlines()[0] if desc else 'it is in the name'
+        # ensure starting lower case
+        desc = desc[0].lower() + desc[1:]
+        return '.SH NAME\n%s \\- %s\n' % (self._bold(prog), desc)
 
     def _mk_description(self, parser):
         desc = parser.description
+        desc = '\n'.join(desc.splitlines()[1:])
         if not desc:
             return ''
-        desc = desc.replace('\n', '\n.br\n')
+        desc = desc.replace('\n\n', '\n.PP\n')
+        # sub-section headings
+        desc = re.sub(r'^\*(.*)\*$', r'.SS \1', desc, flags=re.MULTILINE)
+        # italic commands
+        desc = re.sub(r'^  ([-a-z]*)$', r'.TP\n\\fI\1\\fR', desc, flags=re.MULTILINE)
+        # deindent body text, leave to troff viewer
+        desc = re.sub(r'^      (\S.*)\n', '\\1\n', desc, flags=re.MULTILINE)
+        # format NOTEs as indented paragraphs
+        desc = re.sub(r'^NOTE\n', '.TP\nNOTE\n', desc, flags=re.MULTILINE)
+        # deindent indented paragraphs after heading setup
+        desc = re.sub(r'^  (.*)$', '\\1', desc, flags=re.MULTILINE)
+
         return '.SH DESCRIPTION\n%s\n' % self._markup(desc)
 
     def _mk_footer(self, sections):
@@ -93,6 +116,7 @@ class ManPageFormatter(argparse.HelpFormatter):
     def format_man_page(self, parser):
         page = []
         page.append(self._mk_title(self._prog))
+        page.append(self._mk_name(self._prog, parser.description))
         page.append(self._mk_synopsis(parser))
         page.append(self._mk_description(parser))
         page.append(self._mk_options(parser))
@@ -115,7 +139,14 @@ class ManPageFormatter(argparse.HelpFormatter):
         formatter.add_text(parser.epilog)
 
         # determine help from format above
-        return '.SH OPTIONS\n' + formatter.format_help()
+        help = formatter.format_help()
+        # add spaces after comma delimiters for easier reformatting
+        help = re.sub(r'([a-z]),([a-z])', '\\1, \\2', help)
+        # get proper indentation for argument items
+        help = re.sub(r'^  (\S.*)\n', '.TP\n\\1\n', help, flags=re.MULTILINE)
+        # deindent body text, leave to troff viewer
+        help = re.sub(r'^    (\S.*)\n', '\\1\n', help, flags=re.MULTILINE)
+        return '.SH OPTIONS\n' + help
 
     def _format_action_invocation(self, action):
         if not action.option_strings:
@@ -169,11 +200,13 @@ class RSTManPageFormatter(ManPageFormatter):
         return usage
 
     def _mk_title(self, prog):
-        title = "{0}".format(prog)
-        title += '\n{0}\n\n'.format('=' * len(title))
+        # and an easy to use reference point
+        title = ".. _man_%s:\n\n" % prog.replace(' ', '-')
+        title += "{0}".format(prog)
+        title += '\n{0}\n\n'.format('=' * len(prog))
         return title
 
-    def _make_name(self, parser):
+    def _mk_name(self, prog, desc):
         return ''
 
     def _mk_description(self, parser):
@@ -270,7 +303,8 @@ def cmdline_example_to_rst(src, out=None, ref=None):
             out.write(line[(min(2, len(line) - 1)):])
             continue
         if incodeblock:
-            out.write('  %s' % line)
+            if not line.rstrip().endswith('#% SKIP'):
+                out.write('  %s' % line)
             continue
         if not len(line.strip()):
             continue
